import 'package:flutter/material.dart';
import 'package:yui/component/basic/global.dart';
import 'package:yui/component/basic/theme.dart';

class ThemeColor {
  Color textColor;
  Color disabledTextColor;
  Color backgroundColor;
  Color disabledColor;
  Color borderColor;
  double fontSize;
  Color highlightColor;
  ThemeColor(
      {this.textColor = Colors.white,
      this.fontSize = 18,
      this.highlightColor = YuiColors.highlightColor,
      this.disabledColor = Colors.grey,
      this.disabledTextColor = Colors.grey,
      this.backgroundColor = YuiColors.primaryColor,
      this.borderColor = YuiColors.primaryColor});
}

enum YButtonPlainType {
  /// 突出的按钮
  raised,

  /// 有边框的按钮
  outline,
}

enum YButtonType {
  primary,
  success,
  warning,
  danger,
  info,
}

enum YButtonSize {
  small,
  normal,
}


class YButton extends StatelessWidget {
  final double width;
  final double height;
  final bool loading;
  final String title;
  final Function onPressed;

  /// 目前只用到里面的 color size
  final double fontSize;
  final Color color;
  final YButtonPlainType plain;
  final YButtonType type;
  final bool disabled;
  final bool circle;
  final BorderRadius borderRadius;
  final IconData iconData;

  /// 是否被按下
  bool pressIng;

  YButton(
      {this.height,
      this.width,
      this.title,
      this.onPressed,
      this.iconData,
      this.loading = false,
      this.disabled = false,
      this.plain = YButtonPlainType.raised,
      this.type = YButtonType.primary,
      this.fontSize,
      this.color,
      this.borderRadius,
      this.pressIng = false,
      this.circle = false});
  @override
  Widget build(BuildContext context) {
    MaterialButton button;
    BorderRadius _borderRadius;
    _borderRadius = BorderRadius.all(Radius.circular(circle ? 50 : Yui.rpx(4)));
    if (borderRadius != null) _borderRadius = borderRadius;
    ThemeColor themeColor = _getTheme();
    if (plain == YButtonPlainType.raised) {
      button = RaisedButton(
        shape: RoundedRectangleBorder(
          borderRadius: _borderRadius,
        ),
        animationDuration: Duration(),
        elevation: 0,
        highlightElevation: 0,
        disabledElevation: 0,
        color: themeColor.backgroundColor,
        highlightColor: themeColor.highlightColor,
        child: _getLable(themeColor),
        onPressed: disabled ? null : onPressed,
      );
    } else if (plain == YButtonPlainType.outline) {
      button = OutlineButton(
        shape: RoundedRectangleBorder(
          borderRadius: _borderRadius,
        ),
        borderSide: BorderSide(
          color: themeColor.borderColor,
        ),
        disabledTextColor: themeColor.disabledTextColor,
        color: themeColor.backgroundColor,
        highlightColor: themeColor.highlightColor,
        highlightedBorderColor: themeColor.backgroundColor,
        disabledBorderColor: themeColor.disabledTextColor,
        child: _getLable(themeColor),
        onPressed: disabled ? null : onPressed,
      );
    }

    /// 如果设置可宽高，则包装一层。方便使用
    if (height != null || width != null) {
      return Container(
        child: button,
        width: width,
        height: height,
      );
    }
    return button;
  }

  Widget _getLable(ThemeColor themeColor) {
    Color textColor =
        disabled ? themeColor.disabledTextColor : themeColor.textColor;
    // 计算字体大小，传入的字体大小会覆盖
    double _fontSize = fontSize != null ? fontSize : themeColor.fontSize;
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        Visibility(
          visible: loading,
          child: Container(
            width: _fontSize,
            height: _fontSize,
            child: CircularProgressIndicator(
              strokeWidth: 2,
              backgroundColor: Colors.white,
              valueColor: new AlwaysStoppedAnimation<Color>(
                  color != null ? color : textColor),
            ),
          ),
        ),
        Visibility(
          visible: title != null && loading,
          child: SizedBox(
            width: 5.0,
          ),
        ),
        Visibility(
          visible: iconData != null,
          child: Container(
            margin: EdgeInsets.symmetric(horizontal: 6),
            child: Icon(
              iconData,
              size: _fontSize,
              color: color != null ? color : textColor,
            ),
          ),
        ),
        Visibility(
          visible: title != null,
          child: Text(
            title ?? '',
            style: TextStyle(
              fontSize: _fontSize,
              color: color != null ? color : textColor,
            ),
          ),
        )
      ],
    );
  }

  ThemeColor _getTheme() {
    ThemeColor themeColor = ThemeColor(
      textColor: Colors.white,
      disabledTextColor: Colors.white30,
      borderColor: Colors.grey,
      disabledColor: Colors.grey,
      backgroundColor: Colors.white,
    );
    if (type == YButtonType.primary) {
      themeColor.backgroundColor = YuiColors.primaryColor;
      if (plain == YButtonPlainType.outline) {
        themeColor.textColor = YuiColors.primaryColor;
        themeColor.borderColor = YuiColors.primaryColor;
      }
    } else if (type == YButtonType.success) {
      themeColor.highlightColor = YuiColors.successColor;
      themeColor.backgroundColor = YuiColors.successColor;
      if (plain == YButtonPlainType.outline) {
        themeColor.textColor = YuiColors.successColor;
        themeColor.borderColor = YuiColors.successColor;
      }
    } else if (type == YButtonType.warning) {
      themeColor.highlightColor = YuiColors.warningColor;
      themeColor.backgroundColor = YuiColors.warningColor;
      if (plain == YButtonPlainType.outline) {
        themeColor.textColor = YuiColors.warningColor;
        themeColor.borderColor = YuiColors.warningColor;
      }
    } else if (type == YButtonType.danger) {
      themeColor.highlightColor = YuiColors.dangerColor;
      themeColor.backgroundColor = YuiColors.dangerColor;
      if (plain == YButtonPlainType.outline) {
        themeColor.textColor = YuiColors.dangerColor;
        themeColor.borderColor = YuiColors.dangerColor;
      }
    } else if (type == YButtonType.info) {
      themeColor.highlightColor = YuiColors.infoColor;
      themeColor.backgroundColor = YuiColors.infoColor;
      if (plain == YButtonPlainType.outline) {
        themeColor.textColor = YuiColors.infoColor;
        themeColor.borderColor = YuiColors.infoColor;
      }
    }
    if (plain == YButtonPlainType.outline) {
      themeColor.highlightColor = Colors.white;
    }
    return themeColor;
  }
}
